from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class InlineResponse200_118DataStatus(Enum):
    """An enumeration representing different categories.

    :cvar INPROGRESS: "IN_PROGRESS"
    :vartype INPROGRESS: str
    :cvar ERROR: "ERROR"
    :vartype ERROR: str
    :cvar INPROGRESS1: "IN_PROGRESS"
    :vartype INPROGRESS1: str
    :cvar PARTIAL: "PARTIAL"
    :vartype PARTIAL: str
    :cvar PROCESSED: "PROCESSED"
    :vartype PROCESSED: str
    """

    INPROGRESS = "IN_PROGRESS"
    ERROR = "ERROR"
    INPROGRESS1 = "IN_PROGRESS"
    PARTIAL = "PARTIAL"
    PROCESSED = "PROCESSED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, InlineResponse200_118DataStatus._member_map_.values()
            )
        )


@JsonMap({})
class InlineResponse200_118Data(BaseModel):
    """InlineResponse200_118Data

    :param operation_id: ID of the query request operation. UUID., defaults to None
    :type operation_id: str, optional
    :param partner_merchant_reference: ID of the merchant, defined by the partner.., defaults to None
    :type partner_merchant_reference: str, optional
    :param partner_query_reference: Unique ID of the query request, defined by the partner., defaults to None
    :type partner_query_reference: str, optional
    :param status: Status of the Card Network Lookup Service query. One of the following:\<BR\>* **ERROR** - The query failed.\<BR\>* **IN_PROGRESS** - The query is being processed. Results are not available.\<BR\>* **PARTIAL** - The query process is partially complete. Some results are available.\<BR\>* **PROCESSED** - The query process is complete. All results are available.The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document., defaults to None
    :type status: InlineResponse200_118DataStatus, optional
    """

    def __init__(
        self,
        operation_id: str = None,
        partner_merchant_reference: str = None,
        partner_query_reference: str = None,
        status: InlineResponse200_118DataStatus = None,
    ):
        """InlineResponse200_118Data

        :param operation_id: ID of the query request operation. UUID., defaults to None
        :type operation_id: str, optional
        :param partner_merchant_reference: ID of the merchant, defined by the partner.., defaults to None
        :type partner_merchant_reference: str, optional
        :param partner_query_reference: Unique ID of the query request, defined by the partner., defaults to None
        :type partner_query_reference: str, optional
        :param status: Status of the Card Network Lookup Service query. One of the following:\<BR\>* **ERROR** - The query failed.\<BR\>* **IN_PROGRESS** - The query is being processed. Results are not available.\<BR\>* **PARTIAL** - The query process is partially complete. Some results are available.\<BR\>* **PROCESSED** - The query process is complete. All results are available.The two-letter ISO 3166-1 ALPHA-2 code for the country of the identification document., defaults to None
        :type status: InlineResponse200_118DataStatus, optional
        """
        self.operation_id = self._define_str(
            "operation_id", operation_id, nullable=True
        )
        self.partner_merchant_reference = self._define_str(
            "partner_merchant_reference", partner_merchant_reference, nullable=True
        )
        self.partner_query_reference = self._define_str(
            "partner_query_reference", partner_query_reference, nullable=True
        )
        self.status = (
            self._enum_matching(
                status, InlineResponse200_118DataStatus.list(), "status"
            )
            if status
            else None
        )



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .payment_options import PaymentOptions


class NetworkStatus(Enum):
    """An enumeration representing different categories.

    :cvar APPROVEDBYNETWORK: "approved_by_network"
    :vartype APPROVEDBYNETWORK: str
    :cvar DECLINEDBYNETWORK: "declined_by_network"
    :vartype DECLINEDBYNETWORK: str
    :cvar NOTSENTTONETWORK: "not_sent_to_network"
    :vartype NOTSENTTONETWORK: str
    :cvar REVERSEDAFTERAPPROVAL: "reversed_after_approval"
    :vartype REVERSEDAFTERAPPROVAL: str
    """

    APPROVEDBYNETWORK = "approved_by_network"
    DECLINEDBYNETWORK = "declined_by_network"
    NOTSENTTONETWORK = "not_sent_to_network"
    REVERSEDAFTERAPPROVAL = "reversed_after_approval"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, NetworkStatus._member_map_.values()))


class OutcomePaymentFlowType(Enum):
    """An enumeration representing different categories.

    :cvar DIRECT: "direct"
    :vartype DIRECT: str
    :cvar EWALLET: "eWallet"
    :vartype EWALLET: str
    :cvar EWALLETPAYOUT: "ewallet_payout"
    :vartype EWALLETPAYOUT: str
    :cvar CARD: "card"
    :vartype CARD: str
    :cvar REDIRECTURL: "redirect_url"
    :vartype REDIRECTURL: str
    """

    DIRECT = "direct"
    EWALLET = "eWallet"
    EWALLETPAYOUT = "ewallet_payout"
    CARD = "card"
    REDIRECTURL = "redirect_url"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, OutcomePaymentFlowType._member_map_.values())
        )


class RiskLevel(Enum):
    """An enumeration representing different categories.

    :cvar NORMAL: "normal"
    :vartype NORMAL: str
    :cvar ELEVATED: "elevated"
    :vartype ELEVATED: str
    :cvar HIGHEST: "highest"
    :vartype HIGHEST: str
    :cvar NOTASSESSED: "not_assessed"
    :vartype NOTASSESSED: str
    """

    NORMAL = "normal"
    ELEVATED = "elevated"
    HIGHEST = "highest"
    NOTASSESSED = "not_assessed"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, RiskLevel._member_map_.values()))


@JsonMap({})
class Outcome(BaseModel):
    """The outcome object describes the outcome of the Rapyd Protect risk assessment

    :param network_status: network_status, defaults to None
    :type network_status: NetworkStatus, optional
    :param payment_flow_type: payment_flow_type, defaults to None
    :type payment_flow_type: OutcomePaymentFlowType, optional
    :param payment_options: A payment method type is a type of payment method that any customer can use, for example, ee_mastercard_card, Mastercard for Estonia. When it is added to a customer profile, it becomes a payment method that is specific to that one customer. The name of the payment method type starts with a prefix for the country, the 2-letter ISO 3166-1 alpha-2 code. If the payment method is valid in multiple countries, the prefix is xx_. The payment method type has a suffix with one of the following values - _bank - Bank transfer or bank redirect _card - Credit card, debit card or other card _cash - Cash _ewallet - Local eWallet, defaults to None
    :type payment_options: PaymentOptions, optional
    :param status: Indicates the status of the payment method. One of the following value is 1 means the payment_method_type is Valid, defaults to None
    :type status: str, optional
    :param risk_level: risk_level, defaults to None
    :type risk_level: RiskLevel, optional
    :param seller_message: Message to the merchant, defaults to None
    :type seller_message: str, optional
    """

    def __init__(
        self,
        network_status: NetworkStatus = None,
        payment_flow_type: OutcomePaymentFlowType = None,
        payment_options: PaymentOptions = None,
        status: str = None,
        risk_level: RiskLevel = None,
        seller_message: str = None,
    ):
        """The outcome object describes the outcome of the Rapyd Protect risk assessment

        :param network_status: network_status, defaults to None
        :type network_status: NetworkStatus, optional
        :param payment_flow_type: payment_flow_type, defaults to None
        :type payment_flow_type: OutcomePaymentFlowType, optional
        :param payment_options: A payment method type is a type of payment method that any customer can use, for example, ee_mastercard_card, Mastercard for Estonia. When it is added to a customer profile, it becomes a payment method that is specific to that one customer. The name of the payment method type starts with a prefix for the country, the 2-letter ISO 3166-1 alpha-2 code. If the payment method is valid in multiple countries, the prefix is xx_. The payment method type has a suffix with one of the following values - _bank - Bank transfer or bank redirect _card - Credit card, debit card or other card _cash - Cash _ewallet - Local eWallet, defaults to None
        :type payment_options: PaymentOptions, optional
        :param status: Indicates the status of the payment method. One of the following value is 1 means the payment_method_type is Valid, defaults to None
        :type status: str, optional
        :param risk_level: risk_level, defaults to None
        :type risk_level: RiskLevel, optional
        :param seller_message: Message to the merchant, defaults to None
        :type seller_message: str, optional
        """
        self.network_status = (
            self._enum_matching(network_status, NetworkStatus.list(), "network_status")
            if network_status
            else None
        )
        self.payment_flow_type = (
            self._enum_matching(
                payment_flow_type, OutcomePaymentFlowType.list(), "payment_flow_type"
            )
            if payment_flow_type
            else None
        )
        self.payment_options = self._define_object(payment_options, PaymentOptions)
        self.status = self._define_str("status", status, nullable=True)
        self.risk_level = (
            self._enum_matching(risk_level, RiskLevel.list(), "risk_level")
            if risk_level
            else None
        )
        self.seller_message = self._define_str(
            "seller_message", seller_message, nullable=True
        )



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .payment import Payment
from .order_item_response import OrderItemResponse
from .order_returned_item_response import OrderReturnedItemResponse
from .order_response_status_transitions import OrderResponseStatusTransitions


@JsonMap({"id_": "id"})
class OrderResponse(BaseModel):
    """OrderResponse

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param amount: amount, defaults to None
    :type amount: float, optional
    :param amount_returned: amount_returned, defaults to None
    :type amount_returned: float, optional
    :param payment: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
    :type payment: Payment, optional
    :param created: created, defaults to None
    :type created: int, optional
    :param customer: customer, defaults to None
    :type customer: str, optional
    :param currency: currency, defaults to None
    :type currency: str, optional
    :param email: email, defaults to None
    :type email: str, optional
    :param external_coupon_code: external_coupon_code, defaults to None
    :type external_coupon_code: str, optional
    :param items: items, defaults to None
    :type items: List[OrderItemResponse], optional
    :param metadata: metadata, defaults to None
    :type metadata: dict, optional
    :param returns: A list of the returns charged against this order., defaults to None
    :type returns: List[OrderReturnedItemResponse], optional
    :param shipping_address: shipping_address, defaults to None
    :type shipping_address: dict, optional
    :param status: status, defaults to None
    :type status: str, optional
    :param status_transitions: Indicates the last time in Unix time that the order transitioned to one of the following statuses. A zero value for a status indicates that the order has never transitioned to it., defaults to None
    :type status_transitions: OrderResponseStatusTransitions, optional
    :param updated: updated, defaults to None
    :type updated: int, optional
    :param upstream_id: upstream_id, defaults to None
    :type upstream_id: str, optional
    :param tax_percent: tax_percent, defaults to None
    :type tax_percent: float, optional
    """

    def __init__(
        self,
        id_: str = None,
        amount: float = None,
        amount_returned: float = None,
        payment: Payment = None,
        created: int = None,
        customer: str = None,
        currency: str = None,
        email: str = None,
        external_coupon_code: str = None,
        items: List[OrderItemResponse] = None,
        metadata: dict = None,
        returns: List[OrderReturnedItemResponse] = None,
        shipping_address: dict = None,
        status: str = None,
        status_transitions: OrderResponseStatusTransitions = None,
        updated: int = None,
        upstream_id: str = None,
        tax_percent: float = None,
    ):
        """OrderResponse

        :param id_: id_, defaults to None
        :type id_: str, optional
        :param amount: amount, defaults to None
        :type amount: float, optional
        :param amount_returned: amount_returned, defaults to None
        :type amount_returned: float, optional
        :param payment: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
        :type payment: Payment, optional
        :param created: created, defaults to None
        :type created: int, optional
        :param customer: customer, defaults to None
        :type customer: str, optional
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param email: email, defaults to None
        :type email: str, optional
        :param external_coupon_code: external_coupon_code, defaults to None
        :type external_coupon_code: str, optional
        :param items: items, defaults to None
        :type items: List[OrderItemResponse], optional
        :param metadata: metadata, defaults to None
        :type metadata: dict, optional
        :param returns: A list of the returns charged against this order., defaults to None
        :type returns: List[OrderReturnedItemResponse], optional
        :param shipping_address: shipping_address, defaults to None
        :type shipping_address: dict, optional
        :param status: status, defaults to None
        :type status: str, optional
        :param status_transitions: Indicates the last time in Unix time that the order transitioned to one of the following statuses. A zero value for a status indicates that the order has never transitioned to it., defaults to None
        :type status_transitions: OrderResponseStatusTransitions, optional
        :param updated: updated, defaults to None
        :type updated: int, optional
        :param upstream_id: upstream_id, defaults to None
        :type upstream_id: str, optional
        :param tax_percent: tax_percent, defaults to None
        :type tax_percent: float, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.amount_returned = self._define_number(
            "amount_returned", amount_returned, nullable=True
        )
        self.payment = self._define_object(payment, Payment)
        self.created = self._define_number("created", created, nullable=True)
        self.customer = self._define_str("customer", customer, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.email = self._define_str("email", email, nullable=True)
        self.external_coupon_code = self._define_str(
            "external_coupon_code", external_coupon_code, nullable=True
        )
        self.items = self._define_list(items, OrderItemResponse)
        self.metadata = metadata
        self.returns = self._define_list(returns, OrderReturnedItemResponse)
        self.shipping_address = shipping_address
        self.status = self._define_str("status", status, nullable=True)
        self.status_transitions = self._define_object(
            status_transitions, OrderResponseStatusTransitions
        )
        self.updated = self._define_number("updated", updated, nullable=True)
        self.upstream_id = self._define_str("upstream_id", upstream_id, nullable=True)
        self.tax_percent = self._define_number(
            "tax_percent", tax_percent, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class SubscriptionItemsSubscriptionItemIdBody(BaseModel):
    """SubscriptionItemsSubscriptionItemIdBody

    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param prorate: Determines whether the charge is prorated when a subscription item is switched from one subscription to another in a billing cycle., defaults to None
    :type prorate: bool, optional
    :param proration_date: Indicates the date in the middle of the billing period that is the start of the prorated charges., defaults to None
    :type proration_date: float, optional
    :param quantity: The number of units of the service defined in the plan., defaults to None
    :type quantity: float, optional
    """

    def __init__(
        self,
        metadata: dict = None,
        prorate: bool = None,
        proration_date: float = None,
        quantity: float = None,
    ):
        """SubscriptionItemsSubscriptionItemIdBody

        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param prorate: Determines whether the charge is prorated when a subscription item is switched from one subscription to another in a billing cycle., defaults to None
        :type prorate: bool, optional
        :param proration_date: Indicates the date in the middle of the billing period that is the start of the prorated charges., defaults to None
        :type proration_date: float, optional
        :param quantity: The number of units of the service defined in the plan., defaults to None
        :type quantity: float, optional
        """
        self.metadata = metadata
        self.prorate = prorate
        self.proration_date = self._define_number(
            "proration_date", proration_date, nullable=True
        )
        self.quantity = self._define_number("quantity", quantity, nullable=True)



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1products_package_dimensions import V1productsPackageDimensions


class V1ProductsBodyType(Enum):
    """An enumeration representing different categories.

    :cvar SERVICES: "services"
    :vartype SERVICES: str
    :cvar GOODS: "goods"
    :vartype GOODS: str
    """

    SERVICES = "services"
    GOODS = "goods"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, V1ProductsBodyType._member_map_.values()))


@JsonMap({"id_": "id", "type_": "type"})
class V1ProductsBody(BaseModel):
    """V1ProductsBody

    :param active: Indicates whether the product is currently available for purchase. Default is true., defaults to None
    :type active: bool, optional
    :param attributes: Array of up to 5 alphanumeric strings defined by the merchant. Each string defines the key in a key-value pair in the 'attributes' object in the corresponding 'sku' objects., defaults to None
    :type attributes: List[str], optional
    :param description: Full text description of the product., defaults to None
    :type description: str, optional
    :param id_: Unique string for identification of the product. Legal input includes all the English alphanumeric characters and the underscore (_) character. If the merchant does not define an ID, Rapyd generates it with a string that starts with product_, defaults to None
    :type id_: str, optional
    :param images: An array of images. Each image is a string in Base64 encoding., defaults to None
    :type images: List[str], optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param name: The name of the product or service that is displayed to the customer.
    :type name: str
    :param package_dimensions: Describes the physical size and weight of the product. Relevant when type is goods., defaults to None
    :type package_dimensions: V1productsPackageDimensions, optional
    :param shippable: Indicates whether the product is physically shipped to the customer. Relevant when type is goods. Default is false., defaults to None
    :type shippable: bool, optional
    :param statement_descriptor: A text description that appears in the customer's invoice. Limited to 22 characters. Relevant when type is service., defaults to None
    :type statement_descriptor: str, optional
    :param type_: One of the following - services, goods
    :type type_: V1ProductsBodyType
    :param unit_label: A label that represents units of this product, such as seats, on customers’ invoices. Relevant when type is service., defaults to None
    :type unit_label: str, optional
    """

    def __init__(
        self,
        name: str,
        type_: V1ProductsBodyType,
        active: bool = None,
        attributes: List[str] = None,
        description: str = None,
        id_: str = None,
        images: List[str] = None,
        metadata: dict = None,
        package_dimensions: V1productsPackageDimensions = None,
        shippable: bool = None,
        statement_descriptor: str = None,
        unit_label: str = None,
    ):
        """V1ProductsBody

        :param active: Indicates whether the product is currently available for purchase. Default is true., defaults to None
        :type active: bool, optional
        :param attributes: Array of up to 5 alphanumeric strings defined by the merchant. Each string defines the key in a key-value pair in the 'attributes' object in the corresponding 'sku' objects., defaults to None
        :type attributes: List[str], optional
        :param description: Full text description of the product., defaults to None
        :type description: str, optional
        :param id_: Unique string for identification of the product. Legal input includes all the English alphanumeric characters and the underscore (_) character. If the merchant does not define an ID, Rapyd generates it with a string that starts with product_, defaults to None
        :type id_: str, optional
        :param images: An array of images. Each image is a string in Base64 encoding., defaults to None
        :type images: List[str], optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param name: The name of the product or service that is displayed to the customer.
        :type name: str
        :param package_dimensions: Describes the physical size and weight of the product. Relevant when type is goods., defaults to None
        :type package_dimensions: V1productsPackageDimensions, optional
        :param shippable: Indicates whether the product is physically shipped to the customer. Relevant when type is goods. Default is false., defaults to None
        :type shippable: bool, optional
        :param statement_descriptor: A text description that appears in the customer's invoice. Limited to 22 characters. Relevant when type is service., defaults to None
        :type statement_descriptor: str, optional
        :param type_: One of the following - services, goods
        :type type_: V1ProductsBodyType
        :param unit_label: A label that represents units of this product, such as seats, on customers’ invoices. Relevant when type is service., defaults to None
        :type unit_label: str, optional
        """
        self.active = active
        self.attributes = attributes
        self.description = self._define_str("description", description, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.images = images
        self.metadata = metadata
        self.name = name
        self.package_dimensions = self._define_object(
            package_dimensions, V1productsPackageDimensions
        )
        self.shippable = shippable
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )
        self.type_ = self._enum_matching(type_, V1ProductsBodyType.list(), "type_")
        self.unit_label = self._define_str("unit_label", unit_label, nullable=True)



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    InlineResponse200_90,
    InlineResponse200_91,
    InlineResponse200_92,
    InlineResponse200_93,
    InlineResponse200_94,
    InlineResponse200_95,
    V1VirtualAccountsBody,
    VirtualAccountsTransactionsBody,
    VirtualAccountsVirtualAccountIdBody,
)


class VirtualAccountsService(BaseService):

    @cast_models
    def create_virtual_account(
        self,
        request_body: V1VirtualAccountsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_90:
        """Issue a virtual account number to an existing wallet.

        :param request_body: The request body.
        :type request_body: V1VirtualAccountsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Virtual Accounts
        :rtype: InlineResponse200_90
        """

        Validator(V1VirtualAccountsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/virtual_accounts", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_90._unmap(response)

    @cast_models
    def simulate_bank_transfer_to_virtual_account(
        self,
        request_body: VirtualAccountsTransactionsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_91:
        """Simulate a deposit to a virtual account number that was issued to a wallet. This method is relevant only for testing in the sandbox. The currency of the transfer must be supported by the specific virtual account. This method triggers the Deposit Completed webhook.

        :param request_body: The request body.
        :type request_body: VirtualAccountsTransactionsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Bank transfer details.
        :rtype: InlineResponse200_91
        """

        Validator(VirtualAccountsTransactionsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/virtual_accounts/transactions",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_91._unmap(response)

    @cast_models
    def retrieve_virtual_account(self, virtual_account_id: str) -> InlineResponse200_92:
        """Retrieve a Virtual Account Number object for a wallet.

        :param virtual_account_id: ID of the Virtual Account Number object. String starting with issuing_.
        :type virtual_account_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Virtual Accounts
        :rtype: InlineResponse200_92
        """

        Validator(str).validate(virtual_account_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/virtual_accounts/{{virtualAccountId}}",
                self.get_default_headers(),
            )
            .add_path("virtualAccountId", virtual_account_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_92._unmap(response)

    @cast_models
    def update_requested_currency(
        self,
        virtual_account_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: VirtualAccountsVirtualAccountIdBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_93:
        """Define, change, or cancel the currency to which funds received by a virtual account are converted.<BR>The status of the virtual account must be **ACT** (active) or **PEN** (pending).<BR>To cancel a previously defined currency conversion, set `requested_currency` to **null**.<BR> Prerequisites:<BR>* Create Wallet<BR>* Verify Identity<BR> * Issue Virtual Account to Wallet

        :param request_body: The request body., defaults to None
        :type request_body: VirtualAccountsVirtualAccountIdBody, optional
        :param virtual_account_id: ID of the Virtual Account Number object. String starting with **issuing_**.
        :type virtual_account_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns the requested currency and bank account data.
        :rtype: InlineResponse200_93
        """

        Validator(VirtualAccountsVirtualAccountIdBody).is_optional().validate(
            request_body
        )
        Validator(str).validate(virtual_account_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/virtual_accounts/{{virtualAccountId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("virtualAccountId", virtual_account_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_93._unmap(response)

    @cast_models
    def close_issuing(
        self,
        virtual_account_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_94:
        """Delete a virtual account number of an existing wallet. In order to close a virtual account its status must be ACT.

        :param virtual_account_id: ID of the virtual account. String starting with issuing_.
        :type virtual_account_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Virtual Account
        :rtype: InlineResponse200_94
        """

        Validator(str).validate(virtual_account_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/virtual_accounts/{{virtualAccountId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("virtualAccountId", virtual_account_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_94._unmap(response)

    @cast_models
    def capabilities_of_virtual_accounts(
        self,
        country: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_95:
        """Retrieve a list of the capabilities of virtual accounts you can issue to a Rapyd Wallet.<BR> The list is filtered by country.

        :param country: Two-letter ISO 3166-1 ALPHA-2 code of the country.
        :type country: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns capabilities of virtual accounts.
        :rtype: InlineResponse200_95
        """

        Validator(str).validate(country)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/virtual_accounts/capabilities/{{country}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("country", country)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_95._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InlineResponse200_4DataElements(BaseModel):
    """InlineResponse200_4DataElements

    :param element_name: element_name, defaults to None
    :type element_name: str, optional
    :param verified: verified, defaults to None
    :type verified: bool, optional
    """

    def __init__(self, element_name: str = None, verified: bool = None):
        """InlineResponse200_4DataElements

        :param element_name: element_name, defaults to None
        :type element_name: str, optional
        :param verified: verified, defaults to None
        :type verified: bool, optional
        """
        self.element_name = self._define_str(
            "element_name", element_name, nullable=True
        )
        self.verified = verified



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class ListCountriesResponseLanguages(BaseModel):
    """List of the supported countries.

    :param currency_code: Three-letter ISO 4217 code for the currency., defaults to None
    :type currency_code: str, optional
    :param currency_name: Name of the currency in English., defaults to None
    :type currency_name: str, optional
    :param currency_sign: Unicode symbol for the currency., defaults to None
    :type currency_sign: str, optional
    :param id_: ID of the country., defaults to None
    :type id_: str, optional
    :param iso_alpha2: 2-letter ISO 3166-1 alpha-2 code for the country., defaults to None
    :type iso_alpha2: str, optional
    :param iso_alpha3: 3-letter ISO 3166-1 alpha-2 code for the country. Informational only - not relevant to Rapyd API., defaults to None
    :type iso_alpha3: str, optional
    :param name: Name of the country in English., defaults to None
    :type name: str, optional
    :param phone_code: International telephone prefix for the country., defaults to None
    :type phone_code: str, optional
    """

    def __init__(
        self,
        currency_code: str = None,
        currency_name: str = None,
        currency_sign: str = None,
        id_: str = None,
        iso_alpha2: str = None,
        iso_alpha3: str = None,
        name: str = None,
        phone_code: str = None,
    ):
        """List of the supported countries.

        :param currency_code: Three-letter ISO 4217 code for the currency., defaults to None
        :type currency_code: str, optional
        :param currency_name: Name of the currency in English., defaults to None
        :type currency_name: str, optional
        :param currency_sign: Unicode symbol for the currency., defaults to None
        :type currency_sign: str, optional
        :param id_: ID of the country., defaults to None
        :type id_: str, optional
        :param iso_alpha2: 2-letter ISO 3166-1 alpha-2 code for the country., defaults to None
        :type iso_alpha2: str, optional
        :param iso_alpha3: 3-letter ISO 3166-1 alpha-2 code for the country. Informational only - not relevant to Rapyd API., defaults to None
        :type iso_alpha3: str, optional
        :param name: Name of the country in English., defaults to None
        :type name: str, optional
        :param phone_code: International telephone prefix for the country., defaults to None
        :type phone_code: str, optional
        """
        self.currency_code = self._define_str(
            "currency_code", currency_code, nullable=True
        )
        self.currency_name = self._define_str(
            "currency_name", currency_name, nullable=True
        )
        self.currency_sign = self._define_str(
            "currency_sign", currency_sign, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.iso_alpha2 = self._define_str("iso_alpha2", iso_alpha2, nullable=True)
        self.iso_alpha3 = self._define_str("iso_alpha3", iso_alpha3, nullable=True)
        self.name = self._define_str("name", name, nullable=True)
        self.phone_code = self._define_str("phone_code", phone_code, nullable=True)



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1_addresses_body import V1AddressesBody


class ContactBusinessEntityType(Enum):
    """An enumeration representing different categories.

    :cvar SOLEPROP: "sole_prop"
    :vartype SOLEPROP: str
    :cvar PARTNERSHIP: "partnership"
    :vartype PARTNERSHIP: str
    :cvar COMPANY: "company"
    :vartype COMPANY: str
    :cvar GOVERNMENT: "government"
    :vartype GOVERNMENT: str
    :cvar CHARITY: "charity"
    :vartype CHARITY: str
    :cvar NPO: "NPO"
    :vartype NPO: str
    :cvar ASSOCIATION: "association"
    :vartype ASSOCIATION: str
    :cvar TRUST: "trust"
    :vartype TRUST: str
    """

    SOLEPROP = "sole_prop"
    PARTNERSHIP = "partnership"
    COMPANY = "company"
    GOVERNMENT = "government"
    CHARITY = "charity"
    NPO = "NPO"
    ASSOCIATION = "association"
    TRUST = "trust"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, ContactBusinessEntityType._member_map_.values())
        )


@JsonMap({"id_": "id"})
class ContactBusiness(BaseModel):
    """ContactBusiness

    :param address: address associated with this specific Rapyd entity Payment/Customer etc..., defaults to None
    :type address: V1AddressesBody, optional
    :param annual_revenue: Annual revenue of the business in US dollars. Maximum value 100000000000000. Decimal., defaults to None
    :type annual_revenue: float, optional
    :param cnae_code: Business activity code of the business, according to the ClassificaÃ§Ã£o Nacional de Atividades EconÃ´micas of Brazil. Alphanumeric string. Maximum 7 characters., defaults to None
    :type cnae_code: str, optional
    :param created_at: Time of creation of the business_details object, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param entity_type: Type of business entity, defaults to None
    :type entity_type: ContactBusinessEntityType, optional
    :param establishment_date: Date that the business was established. Format YYYY-MM-DD, defaults to None
    :type establishment_date: str, optional
    :param id_: ID of the business_details object. String starting with busi_., defaults to None
    :type id_: str, optional
    :param industry_category: Name of the industry that this business belongs to. Required. Alphanumeric string with no special characters., defaults to None
    :type industry_category: str, optional
    :param industry_sub_category: Subcategory of the industry that this business belongs to., defaults to None
    :type industry_sub_category: str, optional
    :param legal_entity_type: Type of legal entity. Alphanumeric string., defaults to None
    :type legal_entity_type: str, optional
    :param name: Business name., defaults to None
    :type name: str, optional
    :param registration_number: Registration number., defaults to None
    :type registration_number: str, optional
    """

    def __init__(
        self,
        address: V1AddressesBody = None,
        annual_revenue: float = None,
        cnae_code: str = None,
        created_at: float = None,
        entity_type: ContactBusinessEntityType = None,
        establishment_date: str = None,
        id_: str = None,
        industry_category: str = None,
        industry_sub_category: str = None,
        legal_entity_type: str = None,
        name: str = None,
        registration_number: str = None,
    ):
        """ContactBusiness

        :param address: address associated with this specific Rapyd entity Payment/Customer etc..., defaults to None
        :type address: V1AddressesBody, optional
        :param annual_revenue: Annual revenue of the business in US dollars. Maximum value 100000000000000. Decimal., defaults to None
        :type annual_revenue: float, optional
        :param cnae_code: Business activity code of the business, according to the ClassificaÃ§Ã£o Nacional de Atividades EconÃ´micas of Brazil. Alphanumeric string. Maximum 7 characters., defaults to None
        :type cnae_code: str, optional
        :param created_at: Time of creation of the business_details object, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param entity_type: Type of business entity, defaults to None
        :type entity_type: ContactBusinessEntityType, optional
        :param establishment_date: Date that the business was established. Format YYYY-MM-DD, defaults to None
        :type establishment_date: str, optional
        :param id_: ID of the business_details object. String starting with busi_., defaults to None
        :type id_: str, optional
        :param industry_category: Name of the industry that this business belongs to. Required. Alphanumeric string with no special characters., defaults to None
        :type industry_category: str, optional
        :param industry_sub_category: Subcategory of the industry that this business belongs to., defaults to None
        :type industry_sub_category: str, optional
        :param legal_entity_type: Type of legal entity. Alphanumeric string., defaults to None
        :type legal_entity_type: str, optional
        :param name: Business name., defaults to None
        :type name: str, optional
        :param registration_number: Registration number., defaults to None
        :type registration_number: str, optional
        """
        self.address = self._define_object(address, V1AddressesBody)
        self.annual_revenue = self._define_number(
            "annual_revenue", annual_revenue, nullable=True
        )
        self.cnae_code = self._define_str("cnae_code", cnae_code, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.entity_type = (
            self._enum_matching(
                entity_type, ContactBusinessEntityType.list(), "entity_type"
            )
            if entity_type
            else None
        )
        self.establishment_date = self._define_str(
            "establishment_date",
            establishment_date,
            nullable=True,
            pattern="^\d{4}-\d{2}-\d{2}$",
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.industry_category = self._define_str(
            "industry_category",
            industry_category,
            nullable=True,
            pattern="^[a-zA-Z0-9]+$",
        )
        self.industry_sub_category = self._define_str(
            "industry_sub_category", industry_sub_category, nullable=True
        )
        self.legal_entity_type = self._define_str(
            "legal_entity_type", legal_entity_type, nullable=True
        )
        self.name = self._define_str("name", name, nullable=True)
        self.registration_number = self._define_str(
            "registration_number", registration_number, nullable=True
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .sku_package_dimensions import SkuPackageDimensions


@JsonMap({"id_": "id"})
class Sku(BaseModel):
    """Sku

    :param active: Indicates whether the product is currently being sold. Default is false., defaults to None
    :type active: bool, optional
    :param attributes: Up to 5 alphanumeric key-value pairs defined by the merchant. Each key must match a string in the attributes list of the corresponding product object., defaults to None
    :type attributes: dict, optional
    :param created_at: Time of creation of this SKU, in Unix time. Response  only., defaults to None
    :type created_at: float, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param id_: ID of the SKU object. Alphanumeric characters and underscores. Defined by the merchant, or a string starting with **sku**., defaults to None
    :type id_: str, optional
    :param image: URL of image associated with the product., defaults to None
    :type image: str, optional
    :param inventory: Contains the following fields: type - One of the following values: finite - Physical objects that can be counted. infinite - Products such as downloadable software. bucket - Products that are sold by measurable quantity. quantity - The number of units available in inventory. Integer. Relevant when type is finite. value - The status of the product on hand in inventory. Relevant when type is bucket. One of the following: in_stock - A normal amount of product is available in inventory. limited - There is enough product in inventory for small orders only. * out_of_stock - No product remains in inventory., defaults to None
    :type inventory: dict, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param package_dimensions: Physical attributes of the SKU item. Contains the following fields, height length weight width These fields are represented as numbers, but it is the responsibility of the merchant to define and interpret the relevant units of length and weight., defaults to None
    :type package_dimensions: SkuPackageDimensions, optional
    :param price: Price of one unit. Decimal., defaults to None
    :type price: float, optional
    :param product: ID of the product that this SKU relates to., defaults to None
    :type product: str, optional
    :param updated_at: Time that this product was last updated, in Unix time. Response only., defaults to None
    :type updated_at: float, optional
    """

    def __init__(
        self,
        active: bool = None,
        attributes: dict = None,
        created_at: float = None,
        currency: str = None,
        id_: str = None,
        image: str = None,
        inventory: dict = None,
        metadata: dict = None,
        package_dimensions: SkuPackageDimensions = None,
        price: float = None,
        product: str = None,
        updated_at: float = None,
    ):
        """Sku

        :param active: Indicates whether the product is currently being sold. Default is false., defaults to None
        :type active: bool, optional
        :param attributes: Up to 5 alphanumeric key-value pairs defined by the merchant. Each key must match a string in the attributes list of the corresponding product object., defaults to None
        :type attributes: dict, optional
        :param created_at: Time of creation of this SKU, in Unix time. Response  only., defaults to None
        :type created_at: float, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param id_: ID of the SKU object. Alphanumeric characters and underscores. Defined by the merchant, or a string starting with **sku**., defaults to None
        :type id_: str, optional
        :param image: URL of image associated with the product., defaults to None
        :type image: str, optional
        :param inventory: Contains the following fields: type - One of the following values: finite - Physical objects that can be counted. infinite - Products such as downloadable software. bucket - Products that are sold by measurable quantity. quantity - The number of units available in inventory. Integer. Relevant when type is finite. value - The status of the product on hand in inventory. Relevant when type is bucket. One of the following: in_stock - A normal amount of product is available in inventory. limited - There is enough product in inventory for small orders only. * out_of_stock - No product remains in inventory., defaults to None
        :type inventory: dict, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param package_dimensions: Physical attributes of the SKU item. Contains the following fields, height length weight width These fields are represented as numbers, but it is the responsibility of the merchant to define and interpret the relevant units of length and weight., defaults to None
        :type package_dimensions: SkuPackageDimensions, optional
        :param price: Price of one unit. Decimal., defaults to None
        :type price: float, optional
        :param product: ID of the product that this SKU relates to., defaults to None
        :type product: str, optional
        :param updated_at: Time that this product was last updated, in Unix time. Response only., defaults to None
        :type updated_at: float, optional
        """
        self.active = active
        self.attributes = attributes
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.image = self._define_str("image", image, nullable=True)
        self.inventory = inventory
        self.metadata = metadata
        self.package_dimensions = self._define_object(
            package_dimensions, SkuPackageDimensions
        )
        self.price = self._define_number("price", price, nullable=True)
        self.product = self._define_str("product", product, nullable=True)
        self.updated_at = self._define_number("updated_at", updated_at, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1skussku_id_inventory import V1skusskuIdInventory
from .v1skussku_id_package_dimensions import V1skusskuIdPackageDimensions


@JsonMap({})
class V1SkusBody(BaseModel):
    """V1SkusBody

    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer.
    :type currency: str
    :param inventory: inventory object {quantity, type, value}
    :type inventory: V1skusskuIdInventory
    :param price: The amount of the price. Decimal.
    :type price: float
    :param product: ID of the product that this SKU relates to. The type field of the product must be set to goods.
    :type product: str
    :param active: Indicates whether the product is currently available for purchase., defaults to None
    :type active: bool, optional
    :param attributes: Array of alphanumeric key-value pairs defined by the merchant. Each key must match a string in the "attributes" list of the corresponding "product" object., defaults to None
    :type attributes: List[str], optional
    :param image: URL of image associated with the product., defaults to None
    :type image: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param package_dimensions: Physical attributes of the SKU item. Object containing the following fields - height, length, weight, width, defaults to None
    :type package_dimensions: V1skusskuIdPackageDimensions, optional
    """

    def __init__(
        self,
        currency: str,
        inventory: V1skusskuIdInventory,
        price: float,
        product: str,
        active: bool = None,
        attributes: List[str] = None,
        image: str = None,
        metadata: dict = None,
        package_dimensions: V1skusskuIdPackageDimensions = None,
    ):
        """V1SkusBody

        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer.
        :type currency: str
        :param inventory: inventory object {quantity, type, value}
        :type inventory: V1skusskuIdInventory
        :param price: The amount of the price. Decimal.
        :type price: float
        :param product: ID of the product that this SKU relates to. The type field of the product must be set to goods.
        :type product: str
        :param active: Indicates whether the product is currently available for purchase., defaults to None
        :type active: bool, optional
        :param attributes: Array of alphanumeric key-value pairs defined by the merchant. Each key must match a string in the "attributes" list of the corresponding "product" object., defaults to None
        :type attributes: List[str], optional
        :param image: URL of image associated with the product., defaults to None
        :type image: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param package_dimensions: Physical attributes of the SKU item. Object containing the following fields - height, length, weight, width, defaults to None
        :type package_dimensions: V1skusskuIdPackageDimensions, optional
        """
        self.currency = self._define_str(
            "currency",
            currency,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.inventory = self._define_object(inventory, V1skusskuIdInventory)
        self.price = price
        self.product = product
        self.active = active
        self.attributes = attributes
        self.image = self._define_str("image", image, nullable=True)
        self.metadata = metadata
        self.package_dimensions = self._define_object(
            package_dimensions, V1skusskuIdPackageDimensions
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class InlineResponse200_67Data(BaseModel):
    """InlineResponse200_67Data

    :param id_: discount id, defaults to None
    :type id_: str, optional
    :param deleted: deleted, defaults to None
    :type deleted: bool, optional
    """

    def __init__(self, id_: str = None, deleted: bool = None):
        """InlineResponse200_67Data

        :param id_: discount id, defaults to None
        :type id_: str, optional
        :param deleted: deleted, defaults to None
        :type deleted: bool, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.deleted = deleted



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class IssuingCardsBody(BaseModel):
    """IssuingCardsBody

    :param card_program: ID of the card program that the card is issued from. String starting with **cardprog_**.
    :type card_program: str
    :param country: Two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
    :type country: str, optional
    :param ewallet_contact: ID of the wallet contact that the card is assigned to. String starting with **cont_**.
    :type ewallet_contact: str
    :param expiration_month: Expiration month of the card., defaults to None
    :type expiration_month: str, optional
    :param expiration_year: Expiration year of the card., defaults to None
    :type expiration_year: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    """

    def __init__(
        self,
        card_program: str,
        ewallet_contact: str,
        country: str = None,
        expiration_month: str = None,
        expiration_year: str = None,
        metadata: dict = None,
    ):
        """IssuingCardsBody

        :param card_program: ID of the card program that the card is issued from. String starting with **cardprog_**.
        :type card_program: str
        :param country: Two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
        :type country: str, optional
        :param ewallet_contact: ID of the wallet contact that the card is assigned to. String starting with **cont_**.
        :type ewallet_contact: str
        :param expiration_month: Expiration month of the card., defaults to None
        :type expiration_month: str, optional
        :param expiration_year: Expiration year of the card., defaults to None
        :type expiration_year: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        """
        self.card_program = card_program
        self.country = self._define_str("country", country, nullable=True)
        self.ewallet_contact = ewallet_contact
        self.expiration_month = self._define_str(
            "expiration_month", expiration_month, nullable=True
        )
        self.expiration_year = self._define_str(
            "expiration_year", expiration_year, nullable=True
        )
        self.metadata = metadata



from __future__ import annotations
from enum import Enum
from typing import Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel
from .entity_type import EntityType
from .beneficiary import Beneficiary
from .sender import Sender


class V1PayoutsBodyBeneficiaryGuard(OneOfBaseModel):
    class_list = {"str": str, "Beneficiary": Beneficiary}


V1PayoutsBodyBeneficiary = Union[str, Beneficiary]


class V1PayoutsBodyBeneficiaryEntityType(Enum):
    """An enumeration representing different categories.

    :cvar INDIVIDUAL: "individual"
    :vartype INDIVIDUAL: str
    :cvar COMPANY: "company"
    :vartype COMPANY: str
    """

    INDIVIDUAL = "individual"
    COMPANY = "company"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                V1PayoutsBodyBeneficiaryEntityType._member_map_.values(),
            )
        )


class V1PayoutsBodySenderGuard(OneOfBaseModel):
    class_list = {"str": str, "Sender": Sender}


V1PayoutsBodySender = Union[str, Sender]


@JsonMap({})
class V1PayoutsBody(BaseModel):
    """V1PayoutsBody

    :param beneficiary: String starting with **beneficiary_** or the object describing the beneficiary.
    :type beneficiary: V1PayoutsBodyBeneficiary
    :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase., defaults to None
    :type beneficiary_country: str, optional
    :param beneficiary_entity_type: Type of entity for the beneficiary. One of the following: **individual** \| **company**
    :type beneficiary_entity_type: V1PayoutsBodyBeneficiaryEntityType
    :param confirm_automatically: Determines whether completion of the payout requires confirmation of the FX rate. Relevant to payouts with foreign exchange. Default is false., defaults to None
    :type confirm_automatically: bool, optional
    :param converstion_rate: conversion rate, defaults to None
    :type converstion_rate: float, optional
    :param description: Description of the payout transaction., defaults to None
    :type description: str, optional
    :param ewallet: ID of the wallet that the money is transferred from. String starting with **ewallet_**., defaults to None
    :type ewallet: str, optional
    :param expiration: Determines the day the payout expires, in Unix time. The payout must be completed before the start of this day. Relevant to cash payouts., defaults to None
    :type expiration: str, optional
    :param location: Location of the payout transaction., defaults to None
    :type location: str, optional
    :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payout_amount: Amount of the payout, in units of the currency defined in `payout_currency`. Mandatory when `sender_amount` is not used. Decimal.
    :type payout_amount: float
    :param payout_currency: payout_currency
    :type payout_currency: str
    :param payout_method_type: The type of payout method. The two-letter prefix must match the beneficiary country code. Required when the beneficiary does not have a default payout method type., defaults to None
    :type payout_method_type: str, optional
    :param payout_options: Payout options, defaults to None
    :type payout_options: dict, optional
    :param sender: String starting with **sender_** or the object describing the sender.
    :type sender: V1PayoutsBodySender
    :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code.
    :type sender_country: str
    :param sender_currency: sender_currency
    :type sender_currency: str
    :param sender_entity_type: sender_entity_type
    :type sender_entity_type: EntityType
    :param statement_descriptor: A statement that includes the reason for the payout. Limited to 35 characters., defaults to None
    :type statement_descriptor: str, optional
    """

    def __init__(
        self,
        beneficiary: V1PayoutsBodyBeneficiary,
        beneficiary_entity_type: V1PayoutsBodyBeneficiaryEntityType,
        payout_amount: float,
        payout_currency: str,
        sender: V1PayoutsBodySender,
        sender_country: str,
        sender_currency: str,
        sender_entity_type: EntityType,
        beneficiary_country: str = None,
        confirm_automatically: bool = None,
        converstion_rate: float = None,
        description: str = None,
        ewallet: str = None,
        expiration: str = None,
        location: str = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
        payout_method_type: str = None,
        payout_options: dict = None,
        statement_descriptor: str = None,
    ):
        """V1PayoutsBody

        :param beneficiary: String starting with **beneficiary_** or the object describing the beneficiary.
        :type beneficiary: V1PayoutsBodyBeneficiary
        :param beneficiary_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase., defaults to None
        :type beneficiary_country: str, optional
        :param beneficiary_entity_type: Type of entity for the beneficiary. One of the following: **individual** \| **company**
        :type beneficiary_entity_type: V1PayoutsBodyBeneficiaryEntityType
        :param confirm_automatically: Determines whether completion of the payout requires confirmation of the FX rate. Relevant to payouts with foreign exchange. Default is false., defaults to None
        :type confirm_automatically: bool, optional
        :param converstion_rate: conversion rate, defaults to None
        :type converstion_rate: float, optional
        :param description: Description of the payout transaction., defaults to None
        :type description: str, optional
        :param ewallet: ID of the wallet that the money is transferred from. String starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        :param expiration: Determines the day the payout expires, in Unix time. The payout must be completed before the start of this day. Relevant to cash payouts., defaults to None
        :type expiration: str, optional
        :param location: Location of the payout transaction., defaults to None
        :type location: str, optional
        :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payout_amount: Amount of the payout, in units of the currency defined in `payout_currency`. Mandatory when `sender_amount` is not used. Decimal.
        :type payout_amount: float
        :param payout_currency: payout_currency
        :type payout_currency: str
        :param payout_method_type: The type of payout method. The two-letter prefix must match the beneficiary country code. Required when the beneficiary does not have a default payout method type., defaults to None
        :type payout_method_type: str, optional
        :param payout_options: Payout options, defaults to None
        :type payout_options: dict, optional
        :param sender: String starting with **sender_** or the object describing the sender.
        :type sender: V1PayoutsBodySender
        :param sender_country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code.
        :type sender_country: str
        :param sender_currency: sender_currency
        :type sender_currency: str
        :param sender_entity_type: sender_entity_type
        :type sender_entity_type: EntityType
        :param statement_descriptor: A statement that includes the reason for the payout. Limited to 35 characters., defaults to None
        :type statement_descriptor: str, optional
        """
        self.beneficiary = V1PayoutsBodyBeneficiaryGuard.return_one_of(beneficiary)
        self.beneficiary_country = self._define_str(
            "beneficiary_country", beneficiary_country, nullable=True
        )
        self.beneficiary_entity_type = self._enum_matching(
            beneficiary_entity_type,
            V1PayoutsBodyBeneficiaryEntityType.list(),
            "beneficiary_entity_type",
        )
        self.confirm_automatically = confirm_automatically
        self.converstion_rate = self._define_number(
            "converstion_rate", converstion_rate, nullable=True
        )
        self.description = self._define_str("description", description, nullable=True)
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.expiration = self._define_str("expiration", expiration, nullable=True)
        self.location = self._define_str("location", location, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.payout_amount = payout_amount
        self.payout_currency = self._define_str(
            "payout_currency",
            payout_currency,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.payout_method_type = self._define_str(
            "payout_method_type", payout_method_type, nullable=True
        )
        self.payout_options = payout_options
        self.sender = V1PayoutsBodySenderGuard.return_one_of(sender)
        self.sender_country = sender_country
        self.sender_currency = self._define_str(
            "sender_currency",
            sender_currency,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.sender_entity_type = self._enum_matching(
            sender_entity_type, EntityType.list(), "sender_entity_type"
        )
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"type_": "type"})
class AccountLimitsBody(BaseModel):
    """AccountLimitsBody

    :param account_id: The ID of the account within the wallet., defaults to None
    :type account_id: str, optional
    :param amount: The amount of the limit.
    :type amount: str
    :param currency: Three-letter ISO 4217 code for the currency of an existing account.
    :type currency: str
    :param type_: The limit type.
    :type type_: str
    """

    def __init__(self, amount: str, currency: str, type_: str, account_id: str = None):
        """AccountLimitsBody

        :param account_id: The ID of the account within the wallet., defaults to None
        :type account_id: str, optional
        :param amount: The amount of the limit.
        :type amount: str
        :param currency: Three-letter ISO 4217 code for the currency of an existing account.
        :type currency: str
        :param type_: The limit type.
        :type type_: str
        """
        self.account_id = self._define_str("account_id", account_id, nullable=True)
        self.amount = amount
        self.currency = currency
        self.type_ = type_



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_4_data_compliance_levels import (
    InlineResponse200_4DataComplianceLevels,
)


@JsonMap({})
class InlineResponse200_4Data(BaseModel):
    """InlineResponse200_4Data

    :param compliance_levels: compliance_levels, defaults to None
    :type compliance_levels: List[InlineResponse200_4DataComplianceLevels], optional
    """

    def __init__(
        self, compliance_levels: List[InlineResponse200_4DataComplianceLevels] = None
    ):
        """InlineResponse200_4Data

        :param compliance_levels: compliance_levels, defaults to None
        :type compliance_levels: List[InlineResponse200_4DataComplianceLevels], optional
        """
        self.compliance_levels = self._define_list(
            compliance_levels, InlineResponse200_4DataComplianceLevels
        )



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class V1skusskuIdInventoryType(Enum):
    """An enumeration representing different categories.

    :cvar FINITE: "finite"
    :vartype FINITE: str
    :cvar INFINITE: "infinite"
    :vartype INFINITE: str
    :cvar BUCKET: "bucket"
    :vartype BUCKET: str
    """

    FINITE = "finite"
    INFINITE = "infinite"
    BUCKET = "bucket"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, V1skusskuIdInventoryType._member_map_.values())
        )


class Value(Enum):
    """An enumeration representing different categories.

    :cvar INSTOCK: "in_stock"
    :vartype INSTOCK: str
    :cvar LIMITED: "limited"
    :vartype LIMITED: str
    :cvar OUTOFSTOCK: "out_of_stock"
    :vartype OUTOFSTOCK: str
    """

    INSTOCK = "in_stock"
    LIMITED = "limited"
    OUTOFSTOCK = "out_of_stock"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Value._member_map_.values()))


@JsonMap({"type_": "type"})
class V1skusskuIdInventory(BaseModel):
    """inventory object {quantity, type, value}

    :param type_: type_, defaults to None
    :type type_: V1skusskuIdInventoryType, optional
    :param quantity: quantity, defaults to None
    :type quantity: int, optional
    :param value: value, defaults to None
    :type value: Value, optional
    """

    def __init__(
        self,
        type_: V1skusskuIdInventoryType = None,
        quantity: int = None,
        value: Value = None,
    ):
        """inventory object {quantity, type, value}

        :param type_: type_, defaults to None
        :type type_: V1skusskuIdInventoryType, optional
        :param quantity: quantity, defaults to None
        :type quantity: int, optional
        :param value: value, defaults to None
        :type value: Value, optional
        """
        self.type_ = (
            self._enum_matching(type_, V1skusskuIdInventoryType.list(), "type_")
            if type_
            else None
        )
        self.quantity = self._define_number("quantity", quantity, nullable=True)
        self.value = (
            self._enum_matching(value, Value.list(), "value") if value else None
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardDetailsCardTokenBody(BaseModel):
    """CardDetailsCardTokenBody

    :param card_color: The color of the card, specified by one of the following formats: `hexadecimal value`  `CSS color name` `RGB` `RGBA`  `black`, defaults to None
    :type card_color: str, optional
    :param language: Determines the default language of the hosted page. When this parameter is null, the language of the user's browser is used. If the language of the user's browser cannot be determined, the default language is English, defaults to None
    :type language: str, optional
    :param logo: The URL of the logo image that appears on the card. If the logo is not specified, the name of the merchant appears., defaults to None
    :type logo: str, optional
    :param logo_orientation: The orientation of the logo on the card:   `landscape`  `square` `portrait`      Default value: `landscape` , defaults to None
    :type logo_orientation: str, optional
    """

    def __init__(
        self,
        card_color: str = None,
        language: str = None,
        logo: str = None,
        logo_orientation: str = None,
    ):
        """CardDetailsCardTokenBody

        :param card_color: The color of the card, specified by one of the following formats: `hexadecimal value`  `CSS color name` `RGB` `RGBA`  `black`, defaults to None
        :type card_color: str, optional
        :param language: Determines the default language of the hosted page. When this parameter is null, the language of the user's browser is used. If the language of the user's browser cannot be determined, the default language is English, defaults to None
        :type language: str, optional
        :param logo: The URL of the logo image that appears on the card. If the logo is not specified, the name of the merchant appears., defaults to None
        :type logo: str, optional
        :param logo_orientation: The orientation of the logo on the card:   `landscape`  `square` `portrait`      Default value: `landscape` , defaults to None
        :type logo_orientation: str, optional
        """
        self.card_color = self._define_str("card_color", card_color, nullable=True)
        self.language = self._define_str("language", language, nullable=True)
        self.logo = self._define_str("logo", logo, nullable=True)
        self.logo_orientation = self._define_str(
            "logo_orientation", logo_orientation, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class HostedPageAdditionalResponseCustomElements(BaseModel):
    """Description of the payment transaction.

    :param save_card_default: save_card_default
    :type save_card_default: bool
    :param payment_fees_display: payment_fees_display
    :type payment_fees_display: bool
    :param billing_address_collect: billing_address_collect
    :type billing_address_collect: bool
    :param display_description: display_description
    :type display_description: bool
    :param merchant_currency_only: merchant_currency_only
    :type merchant_currency_only: bool
    :param cardholder_name: cardholder_name
    :type cardholder_name: str
    :param cardholder_preferred_currency: cardholder_preferred_currency, defaults to None
    :type cardholder_preferred_currency: str, optional
    :param dynamic_currency_conversion: dynamic_currency_conversion, defaults to None
    :type dynamic_currency_conversion: bool, optional
    """

    def __init__(
        self,
        save_card_default: bool,
        payment_fees_display: bool,
        billing_address_collect: bool,
        display_description: bool,
        merchant_currency_only: bool,
        cardholder_name: str,
        cardholder_preferred_currency: str = None,
        dynamic_currency_conversion: bool = None,
    ):
        """Description of the payment transaction.

        :param save_card_default: save_card_default
        :type save_card_default: bool
        :param payment_fees_display: payment_fees_display
        :type payment_fees_display: bool
        :param billing_address_collect: billing_address_collect
        :type billing_address_collect: bool
        :param display_description: display_description
        :type display_description: bool
        :param merchant_currency_only: merchant_currency_only
        :type merchant_currency_only: bool
        :param cardholder_name: cardholder_name
        :type cardholder_name: str
        :param cardholder_preferred_currency: cardholder_preferred_currency, defaults to None
        :type cardholder_preferred_currency: str, optional
        :param dynamic_currency_conversion: dynamic_currency_conversion, defaults to None
        :type dynamic_currency_conversion: bool, optional
        """
        self.save_card_default = save_card_default
        self.payment_fees_display = payment_fees_display
        self.billing_address_collect = billing_address_collect
        self.display_description = display_description
        self.merchant_currency_only = merchant_currency_only
        self.cardholder_name = cardholder_name
        self.cardholder_preferred_currency = self._define_str(
            "cardholder_preferred_currency",
            cardholder_preferred_currency,
            nullable=True,
        )
        self.dynamic_currency_conversion = dynamic_currency_conversion



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InlineResponse200_119DataMatchStats(BaseModel):
    """Statistics about the query.

    :param query_match_count: Number of previous queries to the card network termination database that match elements of this merchant query. Does not indicate a registered match. Mastercard only., defaults to None
    :type query_match_count: float, optional
    :param registered_match_count: Total number of registered matches in the card network termination databases with elements shared with this merchant query., defaults to None
    :type registered_match_count: float, optional
    """

    def __init__(
        self, query_match_count: float = None, registered_match_count: float = None
    ):
        """Statistics about the query.

        :param query_match_count: Number of previous queries to the card network termination database that match elements of this merchant query. Does not indicate a registered match. Mastercard only., defaults to None
        :type query_match_count: float, optional
        :param registered_match_count: Total number of registered matches in the card network termination databases with elements shared with this merchant query., defaults to None
        :type registered_match_count: float, optional
        """
        self.query_match_count = self._define_number(
            "query_match_count", query_match_count, nullable=True
        )
        self.registered_match_count = self._define_number(
            "registered_match_count", registered_match_count, nullable=True
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    Customer,
    InlineResponse200_21,
    InlineResponse200_22,
    InlineResponse200_23,
    InvoiceIdPayBody,
    InvoicesInvoiceIdBody,
    V1InvoicesBody,
)


class SubscriptionInvoiceService(BaseService):

    @cast_models
    def list_invoices(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        customer: Customer = None,
        date_: str = None,
        due_date: str = None,
        ending_before: str = None,
        limit: str = None,
        starting_after: str = None,
        subscription: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_21:
        """Retrieve the basic data of an invoice, with individual invoice lines.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param customer: ID of the customer. String starting with **cus_**., defaults to None
        :type customer: Customer, optional
        :param date_: Date that the invoice was created., defaults to None
        :type date_: str, optional
        :param due_date: The date payment is due on this invoice. This value is calculated from the date the invoice is created, plus the number of days specified in the days_until_due field. Format is in Unix time., defaults to None
        :type due_date: str, optional
        :param ending_before: The ID of the invoice created after the last invoice you want to retrieve. card., defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of invoices to return. Range 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param starting_after: The ID of the invoice created before the first invoice you want to retrieve., defaults to None
        :type starting_after: str, optional
        :param subscription: ID of the subscription. String starting with sub_., defaults to None
        :type subscription: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoices Fetched successfully
        :rtype: InlineResponse200_21
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(Customer).is_optional().validate(customer)
        Validator(str).is_optional().validate(date_)
        Validator(str).is_optional().validate(due_date)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(subscription)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/invoices", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("customer", customer)
            .add_query("date", date_)
            .add_query("due_date", due_date)
            .add_query("ending_before", ending_before)
            .add_query("limit", limit)
            .add_query("starting_after", starting_after)
            .add_query("subscription", subscription)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_21._unmap(response)

    @cast_models
    def create_invoice(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: V1InvoicesBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_21:
        """Create an invoice to add a one-time charge to a subscription.<BR>After you create the invoice with this method, create invoice items and assign them to the invoice with Create Invoice Item. <BR> This method triggers the Invoice Created Webhook. This webhook contains the same information as the response. <BR> The following asynchronous webhook provides information about later changes to the Invoice object:Retrieve the basic data of an invoice, with individual invoice lines.<BR> * Invoice Payment Succeeded Webhook <BR>* Invoice Updated Webhook

        :param request_body: The request body., defaults to None
        :type request_body: V1InvoicesBody, optional
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice created successfully
        :rtype: InlineResponse200_21
        """

        Validator(V1InvoicesBody).is_optional().validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/invoices", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_21._unmap(response)

    @cast_models
    def retrieve_invoice(
        self,
        invoice_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Retrieve the basic data of an invoice, with individual invoice lines.

        :param invoice_id: The ID of the invoice that you want to retrieve.
        :type invoice_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice Fetched successfully
        :rtype: InlineResponse200_22
        """

        Validator(str).validate(invoice_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/{{invoiceId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceId", invoice_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)

    @cast_models
    def update_invoice(
        self,
        invoice_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: InvoicesInvoiceIdBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Change or modify an invoice. You can modify the invoice when its status is draft.

        :param request_body: The request body., defaults to None
        :type request_body: InvoicesInvoiceIdBody, optional
        :param invoice_id: The ID of the invoice that you want to updated.
        :type invoice_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice updated successfully
        :rtype: InlineResponse200_22
        """

        Validator(InvoicesInvoiceIdBody).is_optional().validate(request_body)
        Validator(str).validate(invoice_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/{{invoiceId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceId", invoice_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)

    @cast_models
    def delete_invoice(
        self,
        invoice_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_23:
        """Delete an invoice. You can delete an invoice when status is draft.

        :param invoice_id: The ID of the invoice that you want to delete.
        :type invoice_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice deleted successfully
        :rtype: InlineResponse200_23
        """

        Validator(str).validate(invoice_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/{{invoiceId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceId", invoice_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_23._unmap(response)

    @cast_models
    def void_invoice(
        self,
        invoice_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Set an invoice to **void** status. <BR> Void invoices are similar to deleted invoices, but their records are kept for accounting purposes.Retrieve the basic data of an invoice, with individual invoice lines.

        :param invoice_id: The ID of the invoice that you want to void.
        :type invoice_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice successfully set to void
        :rtype: InlineResponse200_22
        """

        Validator(str).validate(invoice_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/{{invoiceId}}/void",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceId", invoice_id)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)

    @cast_models
    def finalize_invoice(
        self,
        invoice_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Finalize an invoice.Invoices are initially created with a draft status, and this is the only state in which an invoice can be finalized. When an invoice is ready to be paid, finalize it. This sets its status to open. Subscriptions automatically create draft invoices during each billing cycle, which are then automatically finalized. When an invoice is finalized, it can no longer be deleted and its final status can be one of the following - Paid Uncollectible* Void. An invoice can be finalized only one time.

        :param invoice_id: ID of the invoice you want to pay.
        :type invoice_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Finalized Invoice,
        :rtype: InlineResponse200_22
        """

        Validator(str).validate(invoice_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/{{invoiceId}}/finalize",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceId", invoice_id)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)

    @cast_models
    def pay_invoice(
        self,
        invoice_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: InvoiceIdPayBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Make a payment against an invoice.

        :param request_body: The request body., defaults to None
        :type request_body: InvoiceIdPayBody, optional
        :param invoice_id: ID of the invoice you want to pay.
        :type invoice_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The Invoice was paid.
        :rtype: InlineResponse200_22
        """

        Validator(InvoiceIdPayBody).is_optional().validate(request_body)
        Validator(str).validate(invoice_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/{{invoiceId}}/pay",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceId", invoice_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)

    @cast_models
    def mark_invoice_uncollectible(
        self,
        invoice_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Set an invoice to **uncollectible** status. <BR> This status indicates that the invoice cannot be paid by the customer.

        :param invoice_id: ID of the invoice you want to mark uncollectible. String starting with **invoice_**.
        :type invoice_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The Invoice was marked as uncollectible.
        :rtype: InlineResponse200_22
        """

        Validator(str).validate(invoice_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/{{invoiceId}}/mark_uncollectible",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceId", invoice_id)
            .serialize()
            .set_method("POST")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)

    @cast_models
    def get_invoice_lines(
        self,
        invoice_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Retrieve invoice lines from an invoice. <BR>Invoice lines are subscription items or invoice items.

        :param invoice_id: ID of the invoice. String starting with **invoice_**.
        :type invoice_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The invoice lines were retrieved.
        :rtype: InlineResponse200_22
        """

        Validator(str).validate(invoice_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/{{invoiceId}}/lines",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceId", invoice_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)

    @cast_models
    def get_upcoming_invoice(
        self,
        customer: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        currency: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Retrieve a preview of the basic data of the upcoming invoice for a customer, with individual invoice lines.<BR> To get the invoice lines of the invoice, use 'Retrieve Invoice Lines from Invoice'.<BR> The response generated cannot be used to generate a payment authorization.

        :param customer: ID of the customer whose invoice you want to retrieve.
        :type customer: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param currency: Three-letter ISO 4217 code for the currency. Required when the invoice is not linked to a specific subscription., defaults to None
        :type currency: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The upcoming invoice was retrieved.
        :rtype: InlineResponse200_22
        """

        Validator(str).validate(customer)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(currency)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/upcoming", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("customer", customer, explode=False)
            .add_query("currency", currency, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)

    @cast_models
    def get_invoice_lines_upcoming_invoice(
        self,
        customer: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        currency: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_22:
        """Retrieve invoice lines from the upcoming invoice for a customer. <BR> If a customer has more than one subscription, this method retrieves the invoice lines of the invoice that is due first.<BR> This method is relevant to invoices that are automatically generated out of subscriptions.

        :param customer: ID of the customer whose invoice you want to retrieve.
        :type customer: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param currency: Three-letter ISO 4217 code for the currency. Required when the invoice is not linked to a specific subscription., defaults to None
        :type currency: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The lines from the upcoming invoice were retrieved.
        :rtype: InlineResponse200_22
        """

        Validator(str).validate(customer)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(currency)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoices/upcoming/lines",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("customer", customer, explode=False)
            .add_query("currency", currency, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_22._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class VirtualAccountsVirtualAccountIdBody(BaseModel):
    """VirtualAccountsVirtualAccountIdBody

    :param requesting_currency: requesting_currency, defaults to None
    :type requesting_currency: str, optional
    """

    def __init__(self, requesting_currency: str = None):
        """VirtualAccountsVirtualAccountIdBody

        :param requesting_currency: requesting_currency, defaults to None
        :type requesting_currency: str, optional
        """
        self.requesting_currency = self._define_str(
            "requesting_currency",
            requesting_currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CardsActivateBody(BaseModel):
    """CardsActivateBody

    :param card: The card token, which is a string starting with 'card_'. If the client is PCI-certified, you can use the actual card number.
    :type card: str
    """

    def __init__(self, card: str):
        """CardsActivateBody

        :param card: The card token, which is a string starting with 'card_'. If the client is PCI-certified, you can use the actual card number.
        :type card: str
        """
        self.card = card



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"date_": "date"})
class DailyRate(BaseModel):
    """Describes currency conversion for payments and payouts. Rapyd uses a snapshot of daily foreign exchange rates fetched at 9 PM UTC. The rate returned includes the FX markup fees.

    :param action_type: The type of transaction that the currency exchange applies to. One of the following - payment, payout, defaults to None
    :type action_type: str, optional
    :param buy_amount: If fixed_side is buy, the value of amount. If fixed_side is sell, the value of buy_currency following the currency exchange transaction. Relevant when amount and `fixed_side` are specified., defaults to None
    :type buy_amount: float, optional
    :param buy_currency: The currency purchased in the currency exchange transaction. Three-letter ISO 4217 code. Uppercase., defaults to None
    :type buy_currency: str, optional
    :param date_: The date when the rate is applicable. Today or earlier. Format is YYYY-MM-DD. Default is today., defaults to None
    :type date_: str, optional
    :param fixed_side: Indicates which currency the rate is fixed for. One of the following : * buy - The currency defined by buy_currency. * sell : The currency defined by sell_currency., defaults to None
    :type fixed_side: str, optional
    :param rate: The exchange rate. Includes FX markup fees., defaults to None
    :type rate: float, optional
    :param sell_amount: If `fixed_side` is **sell**, the value of amount. If `fixed_side` is **buy**, the value of `sell_currency` following the currency exchange transaction. Relevant when `amount` and `fixed_side` are specified., defaults to None
    :type sell_amount: float, optional
    :param sell_currency: Selling currency, defaults to None
    :type sell_currency: str, optional
    """

    def __init__(
        self,
        action_type: str = None,
        buy_amount: float = None,
        buy_currency: str = None,
        date_: str = None,
        fixed_side: str = None,
        rate: float = None,
        sell_amount: float = None,
        sell_currency: str = None,
    ):
        """Describes currency conversion for payments and payouts. Rapyd uses a snapshot of daily foreign exchange rates fetched at 9 PM UTC. The rate returned includes the FX markup fees.

        :param action_type: The type of transaction that the currency exchange applies to. One of the following - payment, payout, defaults to None
        :type action_type: str, optional
        :param buy_amount: If fixed_side is buy, the value of amount. If fixed_side is sell, the value of buy_currency following the currency exchange transaction. Relevant when amount and `fixed_side` are specified., defaults to None
        :type buy_amount: float, optional
        :param buy_currency: The currency purchased in the currency exchange transaction. Three-letter ISO 4217 code. Uppercase., defaults to None
        :type buy_currency: str, optional
        :param date_: The date when the rate is applicable. Today or earlier. Format is YYYY-MM-DD. Default is today., defaults to None
        :type date_: str, optional
        :param fixed_side: Indicates which currency the rate is fixed for. One of the following : * buy - The currency defined by buy_currency. * sell : The currency defined by sell_currency., defaults to None
        :type fixed_side: str, optional
        :param rate: The exchange rate. Includes FX markup fees., defaults to None
        :type rate: float, optional
        :param sell_amount: If `fixed_side` is **sell**, the value of amount. If `fixed_side` is **buy**, the value of `sell_currency` following the currency exchange transaction. Relevant when `amount` and `fixed_side` are specified., defaults to None
        :type sell_amount: float, optional
        :param sell_currency: Selling currency, defaults to None
        :type sell_currency: str, optional
        """
        self.action_type = self._define_str("action_type", action_type, nullable=True)
        self.buy_amount = self._define_number("buy_amount", buy_amount, nullable=True)
        self.buy_currency = self._define_str(
            "buy_currency", buy_currency, nullable=True
        )
        self.date_ = self._define_str("date_", date_, nullable=True)
        self.fixed_side = self._define_str("fixed_side", fixed_side, nullable=True)
        self.rate = self._define_number("rate", rate, nullable=True)
        self.sell_amount = self._define_number(
            "sell_amount", sell_amount, nullable=True
        )
        self.sell_currency = self._define_str(
            "sell_currency", sell_currency, nullable=True
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    Customer,
    InlineResponse200_2,
    InlineResponse200_3,
    PaymentIdCaptureBody,
    PaymentsCompletePaymentBody,
    PaymentsPaymentIdBody,
    V1PaymentsBody,
)


class PaymentService(BaseService):

    @cast_models
    def list_payments(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        created_after: str = None,
        created_before: str = None,
        customer: Customer = None,
        destination_card: str = None,
        ending_before: str = None,
        ewallet: str = None,
        group: bool = None,
        invoice: str = None,
        limit: str = None,
        payment_method: str = None,
        order: str = None,
        starting_after: str = None,
        subscription: str = None,
        merchant_reference_id: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_2:
        """Retrieve a list of all payments that you have created. Filter the list with query parameters.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param created_after: The ID of the payment created before the first payment you want to retrieve. String starting with **payment_**., defaults to None
        :type created_after: str, optional
        :param created_before: The ID of the payment created after the last payment you want to retrieve. String starting with **payment_**., defaults to None
        :type created_before: str, optional
        :param customer: Filters the list for payments related to the specified customer., defaults to None
        :type customer: Customer, optional
        :param destination_card: Filters the list for payments related to the specified destination card., defaults to None
        :type destination_card: str, optional
        :param ending_before: The ID of the payment created after the last payment you want to retrieve. String starting with payment_. Deprecated., defaults to None
        :type ending_before: str, optional
        :param ewallet: Filters the list for payments related to the specified wallet., defaults to None
        :type ewallet: str, optional
        :param group: When true, includes only group payments in the response. When false, excludes group payments from the response. Default is false., defaults to None
        :type group: bool, optional
        :param invoice: Filters according to the invoice. String starting with invoice_., defaults to None
        :type invoice: str, optional
        :param limit: The maximum number of payments to return. Range, 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param payment_method: Filters the list for payments related to the specified payment method., defaults to None
        :type payment_method: str, optional
        :param order: Filters the list for payments related to the specified order., defaults to None
        :type order: str, optional
        :param starting_after: The ID of a payment in the list. The list begins with the payment that was created next after the payment with this ID. Use this filter to get the next page of results. Relevant when ending_before is not used. String starting with payment_., defaults to None
        :type starting_after: str, optional
        :param subscription: Filters the list for payments related to the specified subscription., defaults to None
        :type subscription: str, optional
        :param merchant_reference_id: Merchant-defined ID., defaults to None
        :type merchant_reference_id: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Payments Fetched successfully
        :rtype: InlineResponse200_2
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(created_after)
        Validator(str).is_optional().validate(created_before)
        Validator(Customer).is_optional().validate(customer)
        Validator(str).is_optional().validate(destination_card)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(ewallet)
        Validator(bool).is_optional().validate(group)
        Validator(str).is_optional().validate(invoice)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(payment_method)
        Validator(str).is_optional().validate(order)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(subscription)
        Validator(str).is_optional().validate(merchant_reference_id)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/payments", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("created_after", created_after)
            .add_query("created_before", created_before)
            .add_query("customer", customer)
            .add_query("destination_card", destination_card)
            .add_query("ending_before", ending_before)
            .add_query("ewallet", ewallet)
            .add_query("group", group)
            .add_query("invoice", invoice)
            .add_query("limit", limit)
            .add_query("payment_method", payment_method)
            .add_query("order", order)
            .add_query("starting_after", starting_after)
            .add_query("subscription", subscription)
            .add_query("merchant_reference_id", merchant_reference_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_2._unmap(response)

    @cast_models
    def create_payment(
        self,
        request_body: V1PaymentsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_3:
        """Create a payment

        :param request_body: The request body.
        :type request_body: V1PaymentsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Payment was created
        :rtype: InlineResponse200_3
        """

        Validator(V1PaymentsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/payments", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_3._unmap(response)

    @cast_models
    def retrieve_payment(
        self,
        payment_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_3:
        """Retrieve details of a payment

        :param payment_id: ID of the payment. String starting with **payment_**.
        :type payment_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Payment object
        :rtype: InlineResponse200_3
        """

        Validator(str).validate(payment_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/{{paymentId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("paymentId", payment_id, explode=True)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_3._unmap(response)

    @cast_models
    def update_payment(
        self,
        request_body: PaymentsPaymentIdBody,
        payment_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_3:
        """Change or modify a payment when the status of the payment is ACT (active). You can update additional fields if they are listed under payment_options in the response from Get Payment Method Required Fields and is_updateable is set to true

        :param request_body: The request body.
        :type request_body: PaymentsPaymentIdBody
        :param payment_id: ID of the payment. String starting with payment_.
        :type payment_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payment updated successfully
        :rtype: InlineResponse200_3
        """

        Validator(PaymentsPaymentIdBody).validate(request_body)
        Validator(str).validate(payment_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/{{paymentId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("paymentId", payment_id, explode=True)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_3._unmap(response)

    @cast_models
    def cancel_payment(
        self,
        payment_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_3:
        """Cancel a payment where the status of the payment is ACT. Relevant to payment methods where is_cancelable = true in the response to List Payment Methods by Country. This method triggers the Payment Canceled webhook. This webhook contains the same information as the response. NOTE: If the status is CLO, use the Create Refund method.

        :param payment_id: ID of the payment. String starting with payment_.
        :type payment_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payment canceled successfully
        :rtype: InlineResponse200_3
        """

        Validator(str).validate(payment_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/{{paymentId}}", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("paymentId", payment_id, explode=True)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_3._unmap(response)

    @cast_models
    def capture_payment(
        self,
        payment_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: PaymentIdCaptureBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_3:
        """Capture some or all of a card payment that was previously authorized with `capture` set to **false**.<BR>This method changes the `payment` object status to **CLO** and triggers 'Payment Captured Webhook'. This webhook contains the same information as the response. This method also triggers 'Payment Completed Webhook'. The capture operation is also known as clearing or completion.<BR>The scope of the capture operation depends on the client's pre-authorization permissions. To change these permissions, contact Rapyd Client Support.

        :param request_body: The request body., defaults to None
        :type request_body: PaymentIdCaptureBody, optional
        :param payment_id: ID of the payment. String starting with **payment_**.
        :type payment_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payment captured successfully
        :rtype: InlineResponse200_3
        """

        Validator(PaymentIdCaptureBody).is_optional().validate(request_body)
        Validator(str).validate(payment_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/{{paymentId}}/capture",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("paymentId", payment_id, explode=True)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_3._unmap(response)

    @cast_models
    def complete_payment(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: PaymentsCompletePaymentBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_3:
        """To simulate the completion of a payment by the action of a third party, use this method in the sandbox.<BR> This method changes the payment status to **CLO** (closed), and applies to the **cash**, **bank_redirect**, **bank_transfer**, or **ewallet** payment method types.<BR> This method also returns the 'Payment Completed' Webhook. This webhook contains the same information as the response.<BR> For a card payment:<BR> * Capture after authorization only - Use 'Capture Payment'.<BR> * Simulate 3-D Secure (3DS) verification - See Simulating 3DS Authentication.<BR> **Prerequisites**:<BR> * Create Payment

        :param request_body: The request body., defaults to None
        :type request_body: PaymentsCompletePaymentBody, optional
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: payment completed successfully
        :rtype: InlineResponse200_3
        """

        Validator(PaymentsCompletePaymentBody).is_optional().validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/payments/completePayment",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_3._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class RefundsRefundIdBody(BaseModel):
    """RefundsRefundIdBody

    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: any, optional
    """

    def __init__(self, metadata: any = None):
        """RefundsRefundIdBody

        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: any, optional
        """
        self.metadata = metadata



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class V1ordersorderIdreturnsItemsType(Enum):
    """An enumeration representing different categories.

    :cvar SKU: "sku"
    :vartype SKU: str
    :cvar SHIPPING: "shipping"
    :vartype SHIPPING: str
    :cvar TAX: "tax"
    :vartype TAX: str
    """

    SKU = "sku"
    SHIPPING = "shipping"
    TAX = "tax"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, V1ordersorderIdreturnsItemsType._member_map_.values()
            )
        )


@JsonMap({"type_": "type"})
class V1ordersorderIdreturnsItems(BaseModel):
    """V1ordersorderIdreturnsItems

    :param description: description, defaults to None
    :type description: str, optional
    :param parent: parent, defaults to None
    :type parent: str, optional
    :param type_: type_, defaults to None
    :type type_: V1ordersorderIdreturnsItemsType, optional
    :param quantity: quantity, defaults to None
    :type quantity: float, optional
    :param currency: currency, defaults to None
    :type currency: str, optional
    :param amount: amount, defaults to None
    :type amount: float, optional
    :param order_id: order_id, defaults to None
    :type order_id: str, optional
    """

    def __init__(
        self,
        description: str = None,
        parent: str = None,
        type_: V1ordersorderIdreturnsItemsType = None,
        quantity: float = None,
        currency: str = None,
        amount: float = None,
        order_id: str = None,
    ):
        """V1ordersorderIdreturnsItems

        :param description: description, defaults to None
        :type description: str, optional
        :param parent: parent, defaults to None
        :type parent: str, optional
        :param type_: type_, defaults to None
        :type type_: V1ordersorderIdreturnsItemsType, optional
        :param quantity: quantity, defaults to None
        :type quantity: float, optional
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param amount: amount, defaults to None
        :type amount: float, optional
        :param order_id: order_id, defaults to None
        :type order_id: str, optional
        """
        self.description = self._define_str("description", description, nullable=True)
        self.parent = self._define_str("parent", parent, nullable=True)
        self.type_ = (
            self._enum_matching(type_, V1ordersorderIdreturnsItemsType.list(), "type_")
            if type_
            else None
        )
        self.quantity = self._define_number("quantity", quantity, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.order_id = self._define_str("order_id", order_id, nullable=True)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class SetPinResponseBlockedReason(Enum):
    """An enumeration representing different categories.

    :cvar BLOCKEDREVERSIBLE: "blocked_reversible"
    :vartype BLOCKEDREVERSIBLE: str
    :cvar CANCELED: "canceled"
    :vartype CANCELED: str
    :cvar COMPLIANCE: "compliance"
    :vartype COMPLIANCE: str
    :cvar LOCKEDINCORRECTPIN: "locked_incorrect_pin"
    :vartype LOCKEDINCORRECTPIN: str
    :cvar MIGRATED: "migrated"
    :vartype MIGRATED: str
    :cvar NONE: "none"
    :vartype NONE: str
    :cvar OTHER: "other"
    :vartype OTHER: str
    :cvar REISSUED: "reissued"
    :vartype REISSUED: str
    :cvar SUSPECTEDFRAUD: "suspected_fraud"
    :vartype SUSPECTEDFRAUD: str
    """

    BLOCKEDREVERSIBLE = "blocked_reversible"
    CANCELED = "canceled"
    COMPLIANCE = "compliance"
    LOCKEDINCORRECTPIN = "locked_incorrect_pin"
    MIGRATED = "migrated"
    NONE = "none"
    OTHER = "other"
    REISSUED = "reissued"
    SUSPECTEDFRAUD = "suspected_fraud"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, SetPinResponseBlockedReason._member_map_.values())
        )


class SetPinResponseStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACT: "ACT"
    :vartype ACT: str
    :cvar BLO: "BLO"
    :vartype BLO: str
    :cvar IMP: "IMP"
    :vartype IMP: str
    :cvar INA: "INA"
    :vartype INA: str
    """

    ACT = "ACT"
    BLO = "BLO"
    IMP = "IMP"
    INA = "INA"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, SetPinResponseStatus._member_map_.values()))


@JsonMap({"id_": "id"})
class SetPinResponse(BaseModel):
    """SetPinResponse

    :param activated_at: Time that the card was activated, in Unix time., defaults to None
    :type activated_at: float, optional
    :param assigned_at: Time that the card was assigned to a cardholder, in Unix time., defaults to None
    :type assigned_at: str, optional
    :param bin: Bank Identifier Number for the institution issuing the card., defaults to None
    :type bin: str, optional
    :param blocked_reason: Reason for blocking the card., defaults to None
    :type blocked_reason: SetPinResponseBlockedReason, optional
    :param card_id: ID of the card. String starting with **card_**., defaults to None
    :type card_id: str, optional
    :param card_program: ID of the card program that the card is issued from. String starting with **cardprog_**., defaults to None
    :type card_program: str, optional
    :param card_tracking_id: Reserved., defaults to None
    :type card_tracking_id: str, optional
    :param country_iso_alpha_2: The country where the card is issued. Two-letter ISO 3166-1 ALPHA-2 code., defaults to None
    :type country_iso_alpha_2: str, optional
    :param created_at: Time of creation of the issued card object, in Unix time., defaults to None
    :type created_at: float, optional
    :param cvv: Card security code. only., defaults to None
    :type cvv: float, optional
    :param expiration_month: Expiration month of the card. Two digits. Relevant when the card issuer supports it for the country. only., defaults to None
    :type expiration_month: str, optional
    :param expiration_year: Expiration year of the card. Two digits. Relevant when the card issuer supports it for the country., defaults to None
    :type expiration_year: str, optional
    :param ewallet_contact: Describes the wallet contact that the card is assigned to. String starting with **cont_**. For details about the fields of the 'contact' object, see 'Add Contact to Wallet'in online API reference., defaults to None
    :type ewallet_contact: str, optional
    :param id_: ID of the issued card object. String starting with **ci_**., defaults to None
    :type id_: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: str, optional
    :param status: Status of the card. One of the following:\<BR\>* **ACT** - Active.\<BR\> * **BLO** - Blocked.\<BR\>* **IMP** - Imported in bulk, but not yet personalized.\<BR\>* **INA** - Inactive.\<BR\>, defaults to None
    :type status: SetPinResponseStatus, optional
    :param sub_bin: Two-digit code., defaults to None
    :type sub_bin: str, optional
    """

    def __init__(
        self,
        activated_at: float = None,
        assigned_at: str = None,
        bin: str = None,
        blocked_reason: SetPinResponseBlockedReason = None,
        card_id: str = None,
        card_program: str = None,
        card_tracking_id: str = None,
        country_iso_alpha_2: str = None,
        created_at: float = None,
        cvv: float = None,
        expiration_month: str = None,
        expiration_year: str = None,
        ewallet_contact: str = None,
        id_: str = None,
        metadata: str = None,
        status: SetPinResponseStatus = None,
        sub_bin: str = None,
    ):
        """SetPinResponse

        :param activated_at: Time that the card was activated, in Unix time., defaults to None
        :type activated_at: float, optional
        :param assigned_at: Time that the card was assigned to a cardholder, in Unix time., defaults to None
        :type assigned_at: str, optional
        :param bin: Bank Identifier Number for the institution issuing the card., defaults to None
        :type bin: str, optional
        :param blocked_reason: Reason for blocking the card., defaults to None
        :type blocked_reason: SetPinResponseBlockedReason, optional
        :param card_id: ID of the card. String starting with **card_**., defaults to None
        :type card_id: str, optional
        :param card_program: ID of the card program that the card is issued from. String starting with **cardprog_**., defaults to None
        :type card_program: str, optional
        :param card_tracking_id: Reserved., defaults to None
        :type card_tracking_id: str, optional
        :param country_iso_alpha_2: The country where the card is issued. Two-letter ISO 3166-1 ALPHA-2 code., defaults to None
        :type country_iso_alpha_2: str, optional
        :param created_at: Time of creation of the issued card object, in Unix time., defaults to None
        :type created_at: float, optional
        :param cvv: Card security code. only., defaults to None
        :type cvv: float, optional
        :param expiration_month: Expiration month of the card. Two digits. Relevant when the card issuer supports it for the country. only., defaults to None
        :type expiration_month: str, optional
        :param expiration_year: Expiration year of the card. Two digits. Relevant when the card issuer supports it for the country., defaults to None
        :type expiration_year: str, optional
        :param ewallet_contact: Describes the wallet contact that the card is assigned to. String starting with **cont_**. For details about the fields of the 'contact' object, see 'Add Contact to Wallet'in online API reference., defaults to None
        :type ewallet_contact: str, optional
        :param id_: ID of the issued card object. String starting with **ci_**., defaults to None
        :type id_: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: str, optional
        :param status: Status of the card. One of the following:\<BR\>* **ACT** - Active.\<BR\> * **BLO** - Blocked.\<BR\>* **IMP** - Imported in bulk, but not yet personalized.\<BR\>* **INA** - Inactive.\<BR\>, defaults to None
        :type status: SetPinResponseStatus, optional
        :param sub_bin: Two-digit code., defaults to None
        :type sub_bin: str, optional
        """
        self.activated_at = self._define_number(
            "activated_at", activated_at, nullable=True
        )
        self.assigned_at = self._define_str("assigned_at", assigned_at, nullable=True)
        self.bin = self._define_str("bin", bin, nullable=True)
        self.blocked_reason = (
            self._enum_matching(
                blocked_reason, SetPinResponseBlockedReason.list(), "blocked_reason"
            )
            if blocked_reason
            else None
        )
        self.card_id = self._define_str("card_id", card_id, nullable=True)
        self.card_program = self._define_str(
            "card_program", card_program, nullable=True
        )
        self.card_tracking_id = self._define_str(
            "card_tracking_id", card_tracking_id, nullable=True
        )
        self.country_iso_alpha_2 = self._define_str(
            "country_iso_alpha_2", country_iso_alpha_2, nullable=True
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.cvv = self._define_number("cvv", cvv, nullable=True)
        self.expiration_month = self._define_str(
            "expiration_month", expiration_month, nullable=True
        )
        self.expiration_year = self._define_str(
            "expiration_year", expiration_year, nullable=True
        )
        self.ewallet_contact = self._define_str(
            "ewallet_contact", ewallet_contact, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.metadata = self._define_str("metadata", metadata, nullable=True)
        self.status = (
            self._enum_matching(status, SetPinResponseStatus.list(), "status")
            if status
            else None
        )
        self.sub_bin = self._define_str("sub_bin", sub_bin, nullable=True)
